<?xml version="1.0" encoding="ISO-8859-1"?>
<!--
The XSL file that transforms an Ant script into a HTML documentation.
This transformation is part of AntDoc.

Througout the XSL, the 'context' parameter contains context information about the file currently written in. If it not provided to the template, it means that the file being output is in the root destination directory.
-->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xalan="http://xml.apache.org/xalan" xmlns:redirect="org.apache.xalan.xslt.extensions.Redirect" extension-element-prefixes="redirect">
	<xsl:output method="html" indent="yes" standalone="yes" encoding="ISO-8859-1" omit-xml-declaration="yes"/>
	<!--the output trace level: depending on that level, some messages may be output-->
	<xsl:param name="trace_level" select="'none'"/>
	<!--the URL of the Ant site; may be a file path, and in that case, should be equal to ${ant.home}-->
	<xsl:param name="ant_url">http://ant.apache.org</xsl:param>
	<!--the XML file that contains all the Ant taks references (core and optional)-->
	<xsl:param name="ant_tasks_file">AntTasks.xml</xsl:param>
	<!--
	The URL of the XML source file that is being used as a basic entry. Thus, this may be the XML tasks file, or the Ant buildfile directly.
	-->
	<xsl:param name="xml_source_file_url"/>
	<!--
	If not empty, indicates the only name of the target for which documentation should be performed.
	In that case, the input XML file must be the Ant buildfile that defines the project.
	<font color="red">This option is incompatible with the next <code>single_project_description</code> parameter</font>.
	-->
	<xsl:param name="single_target_name"/>
	<!--
	If not empty, indicates that the only documentation for an Ant buildfile project should be generated.
	In that case, the input XML file must be the Ant buildfile that defines the project.
	<font color="red">This option is incompatible with the preceding <code>single_target_name</code> parameter</font>.
	-->
	<xsl:param name="single_project_description"/>
	<!--the title of the generated HTML pages-->
	<xsl:param name="title" select="'AntDoc'"/>
	<!--the content of the HTML body element, used as a user-supplied welcome page main text: in that case, the default welcome page is not generated-->
	<xsl:param name="welcome_html_content"/>
	<!--the URL of the directory in which the documentation is generated-->
	<xsl:param name="output_path"/>
	<!-- the header text used, and displayed at the top navigation level-->
	<xsl:param name="header_text"/>
	<!-- the footer text used, and displayed at the bottom navigation level-->
	<xsl:param name="footer_text"/>
	<!--the Ant home path-->
	<xsl:param name="ant_home"/>
	<!--the flag that indicates whether Ant runtime part should be generated-->
	<xsl:param name="ant_runtime"/>
	<!--the code base for the applet execution-->
	<xsl:param name="ant_applet_codebase"/>
	<!--the classpath of the applet-->
	<xsl:param name="ant_applet_classname"/>
	<!--
	The fully qualified name of the Java class thet is being used as an XSL transformer factory.
	-->
	<xsl:param name="transformer_factory_class_fqn"/>
	<!--the file path of the AntDoc logo image-->
	<xsl:param name="antdoc_gui_logo_image_file_name"/>
	<!--the name of the AntDoc applet-->
	<xsl:variable name="antdoc_applet_name" select="'AntDocApplet'"/>
	<xsl:variable name="tool_author">Edouard Mercier</xsl:variable>
	<xsl:variable name="tool_author_email">antdoc@free.fr</xsl:variable>
	<xsl:variable name="tool_author_web_site">http://antdoc.free.fr</xsl:variable>
	<xsl:variable name="tool_name">AntDoc</xsl:variable>
	<xsl:variable name="tool_version">0.8g</xsl:variable>
	<xsl:variable name="tool_lastupdate">2003.10.01 21:48:07</xsl:variable>
	<!--the jar of AntDoc for the applet execution: works with SUN Java-Plugin-->
	<xsl:variable name="ant_antdoc_jar" select="'AntDoc.jar'"/>
	<!--the cabinet of AntDoc for the applet execution: works with Microsoft JVM embedded in Internet Explorer-->
	<xsl:variable name="ant_antdoc_cab" select="'AntDoc.cab'"/>
	<!--the name of the generated HTML menu file; the menu file is the frame content at the left-->
	<xsl:variable name="menu_file">menu.html</xsl:variable>
	<!--the name of the generated HTML welcome file; the welcome file is the first page shown in the frame content at the right-->
	<xsl:variable name="welcome_file">welcome.html</xsl:variable>
	<!--the name of the generated HTML task usafe file, which displays the uses of the Ant tasks -->
	<xsl:variable name="task_usage_file">task-usage.html</xsl:variable>
	<!--the name of the generated HTML about file-->
	<xsl:variable name="about_file">about.html</xsl:variable>
	<!--the name of the HTML file that contains all Ant targets and that is positionned on the bottom left frame-->
	<xsl:variable name="target_overview_file">overview-frame.html</xsl:variable>
	<!--the name of frame target at the left side-->
	<xsl:variable name="content_target">thecontent</xsl:variable>
	<!--the name of the targets frame at the left bottom-->
	<xsl:variable name="targets_target">thetargets</xsl:variable>
	<!--The color used in order to render the XML extra comments-->
	<xsl:variable name="xml_comments_color" select="'#708080'"/>
	<!--The name of the CSS class that is used when generating the information contained in the Ant buildfile XML comments-->
	<xsl:variable name="antdoc_css_class_name" select="'antdoc'"/>
	<!--this key will be used in order to sort the tasks, on the task usage HTML page-->
	<xsl:key name="task_key" match="task" use="@name"/>
	<xsl:template match="/Scripts">
		<xsl:if test="$trace_level = 'debug'">
			<xsl:message>
				<xsl:text>AntDoc output directory=</xsl:text>
				<xsl:call-template name="_print_directory_output_path"/>
			</xsl:message>
			<xsl:message>
				<xsl:text>Ant task file=</xsl:text>
				<xsl:value-of select="$ant_tasks_file"/>
			</xsl:message>
			<xsl:message>
				<xsl:text>AntDoc script file=</xsl:text>
				<xsl:value-of select="$xml_source_file_url"/>
			</xsl:message>
			<xsl:message>
				<xsl:text>AntDoc script file content=</xsl:text>
				<xsl:copy-of select="document($xml_source_file_url)"/>
			</xsl:message>
			<xsl:message>
				<xsl:text>AntDoc welcome HTML file=</xsl:text>
				<xsl:value-of select="$welcome_html_content"/>
			</xsl:message>
		</xsl:if>
		<xsl:call-template name="write_index"/>
		<xsl:call-template name="write_menu"/>
		<xsl:call-template name="write_overview"/>
		<xsl:call-template name="write_welcome"/>
		<xsl:call-template name="write_task_usage"/>
		<xsl:call-template name="write_about"/>
		<xsl:for-each select="Script">
			<xsl:variable name="ant_script_file" select="string(@filtered)"/>
			<xsl:variable name="ant_script_filename" select="string(@filename)"/>
			<xsl:variable name="ant_project_name" select="document($ant_script_file)/project/@name"/>
			<xsl:variable name="ant_script_local_file" select="@local"/>
			<xsl:variable name="ant_script_local_javascript_file" select="@local_javascript"/>
			<xsl:variable name="doc_file">
				<xsl:call-template name="_print_directory_output_path">
					<xsl:with-param name="project_name" select="$ant_project_name"/>
				</xsl:call-template>
				<xsl:text>/</xsl:text>
				<xsl:call-template name="_print_project_file_name">
					<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:result-document href="{$doc_file}">
				<xsl:apply-templates select="document($ant_script_file)/project">
					<xsl:with-param name="ant_script_filename" select="$ant_script_filename"/>
					<xsl:with-param name="ant_script_local_file" select="$ant_script_local_file"/>
					<xsl:with-param name="ant_script_local_javascript_file" select="$ant_script_local_javascript_file"/>
				</xsl:apply-templates>
			</xsl:result-document>
			<xsl:variable name="target_summary_file">
				<xsl:call-template name="_print_directory_output_path">
					<xsl:with-param name="project_name" select="$ant_project_name"/>
				</xsl:call-template>
				<xsl:text>/</xsl:text>
				<xsl:call-template name="_print_ant_target_overview_html_file_name">
					<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:result-document href="{$target_summary_file}">
				<xsl:variable name="header">
					<xsl:value-of select="$ant_project_name"/>
					<xsl:text> Targets</xsl:text>
				</xsl:variable>
				<xsl:variable name="targets">
					<xsl:for-each select="document($ant_script_file)/project/target">
						<xsl:variable name="target_href">
							<xsl:call-template name="_print_project_target_href">
								<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
								<xsl:with-param name="target_name" select="string(@name)"/>
							</xsl:call-template>
						</xsl:variable>
						<Target name="{@name}" href="{$target_href}"/>
					</xsl:for-each>
				</xsl:variable>
				<xsl:call-template name="print_targets_overview">
					<xsl:with-param name="title" select="$ant_project_name"/>
					<xsl:with-param name="header" select="$header"/>
					<xsl:with-param name="targets" select="$targets"/>
				</xsl:call-template>
			</xsl:result-document>
		</xsl:for-each>
	</xsl:template>
	<!--
	The handling of the Ant project element.
	The 'ant_script_filename', 'ant_script_local_file' and 'ant_script_local_javascript_file' parameters may be provided and be equal to the original Ant script file.
	-->
	<xsl:template match="/project">
		<xsl:param name="ant_script_filename"/>
		<xsl:param name="ant_script_local_file"/>
		<xsl:param name="ant_script_local_javascript_file"/>
		<xsl:choose>
			<xsl:when test="$single_target_name">
				<xsl:call-template name="print_documentation_for_a_single_target"/>
			</xsl:when>
			<xsl:when test="$single_project_description">
				<xsl:call-template name="print_documentation_for_a_project_description"/>
			</xsl:when>
			<xsl:otherwise>
				<html>
					<head>
						<xsl:call-template name="_print_link_and_meta_head">
							<xsl:with-param name="relative_path" select="'..'"/>
						</xsl:call-template>
						<title>
							<xsl:value-of select="@name"/> AntDoc</title>
						<script language="javascript">
function run_ant_target(target_name)
{
	if (document.<xsl:value-of select="$antdoc_applet_name"/> != null)
	{
		document.<xsl:value-of select="$antdoc_applet_name"/>.setTarget(target_name);
	}
}
</script>
					</head>
					<body bgcolor="white">
						<xsl:call-template name="_print_navigation">
							<xsl:with-param name="text" select="$header_text"/>
							<xsl:with-param name="location" select="'script'"/>
							<xsl:with-param name="project" select="."/>
						</xsl:call-template>
						<hr/>
						<xsl:call-template name="_print_project_description">
							<xsl:with-param name="ant_script_filename" select="$ant_script_filename"/>
						</xsl:call-template>
						<xsl:for-each select="target">
							<xsl:variable name="target_file_path">
								<xsl:call-template name="_print_target_html_file_name">
									<xsl:with-param name="target_name" select="@name"/>
									<xsl:with-param name="project_name" select="../@name"/>
								</xsl:call-template>
							</xsl:variable>
							<xsl:call-template name="write_target"/>
						</xsl:for-each>
						<hr/>
						<xsl:variable name="ant_runtime_anchor_name" select="'_ant_runtime_'"/>
						<xsl:if test="$ant_runtime">
							<a name="{$ant_runtime_anchor_name}"/>
							<table>
								<tr>
									<td>
										<applet id="{$antdoc_applet_name}" name="{$antdoc_applet_name}" width="600" height="40" codebase="{$ant_applet_codebase}" archive="{$ant_antdoc_jar}" code="{$ant_applet_classname}" alt="The applet that connects to AntDoc GUI">
											<param name="cabbase" value="{$ant_antdoc_cab}"/>
											<param name="ant.home" value="{$ant_home}"/>
											<param name="ant.project.file" value="{$ant_script_local_file}"/>
											<param name="ant.project.name" value="{/project/@name}"/>
										</applet>
									</td>
								</tr>
							</table>
							<hr/>
						</xsl:if>
						<a name="summary"/>
						<table border="1" cellpadding="3" cellspacing="0" width="100%">
							<tr class="TableHeadingColor" bgcolor="#CCCCFF">
								<td>
									<xsl:attribute name="colspan"><xsl:choose><xsl:when test="$ant_runtime">4</xsl:when><xsl:otherwise>3</xsl:otherwise></xsl:choose></xsl:attribute>
									<font size="+2">
										<b>Target Summary</b>
									</font>
								</td>
							</tr>
							<tr class="TableSubHeadingColor" bgcolor="#EEEEFF">
								<th>Depends<br/>
									<font size="-2">(the <code>depends</code> attribute)</font>
								</th>
								<th>Runtime<br/>
									<font size="-2">(the target executed at runtime)</font>
								</th>
								<xsl:if test="$ant_runtime">
									<th>Run<br/>
										<font size="-2">(via AntDocGUI)</font>
									</th>
								</xsl:if>
								<th>Description</th>
							</tr>
							<xsl:for-each select="target">
								<xsl:sort order="ascending" select="@name"/>
								<tr class="TableRowColor" bgcolor="white">
									<td align="left" valign="top" width="1%">
										<font size="-1">
											<pre>
												<xsl:variable name="result">
													<xsl:call-template name="_print_static_target_dependencies">
														<xsl:with-param name="project" select=".."/>
														<xsl:with-param name="target_name" select="@name"/>
													</xsl:call-template>
												</xsl:variable>
												<xsl:choose>
													<xsl:when test="string-length($result) > 0">
														<xsl:copy-of select="$result"/>
													</xsl:when>
													<xsl:otherwise>&#160;</xsl:otherwise>
												</xsl:choose>
											</pre>
										</font>
									</td>
									<td align="right" valign="top" width="2%">
										<font size="-1">
											<code>
												<xsl:call-template name="_print_runtime_target_dependencies">
													<xsl:with-param name="project" select=".."/>
													<xsl:with-param name="target_name" select="@name"/>
												</xsl:call-template>
											</code>
										</font>
									</td>
									<xsl:if test="$ant_runtime">
										<td>
											<xsl:call-template name="_print_ant_runtime_a">
												<xsl:with-param name="target_name" select="@name"/>
											</xsl:call-template>
										</td>
									</xsl:if>
									<td>
										<code>
											<b>
												<xsl:variable name="text">
													<xsl:call-template name="_print_project_target_a">
														<xsl:with-param name="project" select=".."/>
														<xsl:with-param name="project_name" select="../@name"/>
														<xsl:with-param name="target_name" select="string(@name)"/>
													</xsl:call-template>
												</xsl:variable>
												<xsl:choose>
													<xsl:when test="count(@description) > 0">
														<font size="+1">
															<xsl:copy-of select="$text"/>
														</font>
													</xsl:when>
													<xsl:otherwise>
														<xsl:copy-of select="$text"/>
													</xsl:otherwise>
												</xsl:choose>
											</b>
										</code>
										<br/>
										<font size="+1">
											<xsl:call-template name="_print_summary_target_description"/>
										</font>
									</td>
								</tr>
							</xsl:for-each>
						</table>
						<xsl:text>&#160;</xsl:text>
						<a name="property"/>
						<table border="1" cellpadding="3" cellspacing="0" width="100%">
							<tr class="TableHeadingColor" bgcolor="#CCCCFF">
								<td>
									<font size="+2">
										<b>Top Property</b>
									</font>
								</td>
							</tr>
						</table>
						<table>
							<tr class="TableSubHeadingColor" bgcolor="#EEEEFF">
								<td>
									<pre>
										<xsl:for-each select="/project/property">
											<xsl:sort order="ascending" select="@name"/>
											<xsl:apply-templates select="." mode="htmlout"/>
										</xsl:for-each>
									</pre>
								</td>
							</tr>
						</table>
						<xsl:text>&#160;</xsl:text>
						<a name="detail"/>
						<table border="1" cellpadding="3" cellspacing="0" width="100%">
							<tr class="TableHeadingColor" bgcolor="#CCCCFF">
								<td>
									<font size="+2">
										<b>Target Detail</b>
									</font>
								</td>
							</tr>
						</table>
						<!--xsl:for-each select="target">
							<xsl:sort order="ascending" select="@name"/>
							<a>
								<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
							</a>
							<xsl:call-template name="_print_full_target_details"/>
							<xsl:if test="position() != last()">
								<hr/>
							</xsl:if>
						</xsl:for-each-->
						<hr/>
						<xsl:call-template name="_print_navigation">
							<xsl:with-param name="text" select="$footer_text"/>
							<xsl:with-param name="location" select="'script'"/>
							<xsl:with-param name="project" select="."/>
						</xsl:call-template>
						<xsl:call-template name="_print_copyright_footer"/>
					</body>
				</html>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
	Prints the description part of an Ant project.
	The current node should be the "project" element for which the description generation is being asked for.
	The 'ant_script_filename' parameter is expected and should be equal to the underlying Ant buildfile path or URL.
	-->
	<xsl:template name="_print_project_description">
		<xsl:param name="ant_script_filename"/>
		<h2>
			<xsl:text>Ant project </xsl:text>
			<code>
				<xsl:value-of select="@name"/>
			</code>
			<xsl:text> (file: </xsl:text>
			<code>
				<xsl:value-of select="$ant_script_filename"/>
			</code>
			<xsl:text>)</xsl:text>
		</h2>
		<dl>
			<dt>
				<b>Initial base directory:</b>
				<dd>
					<code>
						<b>
							<xsl:value-of select="@basedir"/>
						</b>
					</code>
				</dd>
			</dt>
		</dl>
		<dl>
			<dt>
				<b>Default target:</b>
				<dd>
					<code>
						<b>
							<a>
								<xsl:attribute name="href">#<xsl:value-of select="@default"/></xsl:attribute>
								<xsl:value-of select="@default"/>
							</a>
						</b>
					</code>
				</dd>
			</dt>
		</dl>
		<hr/>
		<xsl:if test="count(description) > 0">
			<span class="{$antdoc_css_class_name}">
				<p>
					<xsl:value-of select="description" disable-output-escaping="yes"/>
				</p>
				<xsl:if test="description/preceding-sibling::node()[normalize-space()][1][self::comment()]">
					<p>
						<font color="{$xml_comments_color}">
							<xsl:value-of select="description/preceding-sibling::node()[normalize-space()][1][self::comment()]" disable-output-escaping="yes"/>
						</font>
					</p>
				</xsl:if>
			</span>
		</xsl:if>
	</xsl:template>
	<!--
	Prints all available information concerning an Ant target.
	The current node should be this target.
	-->
	<xsl:template name="_print_full_target_details">
		<xsl:variable name="task_header">
			<h2>
				<font size="-1">
					<xsl:value-of select="../@name"/>
				</font>
				<br/>Target <xsl:value-of select="@name"/>
			</h2>
			<pre>
				<xsl:call-template name="_print_static_target_dependencies">
					<xsl:with-param name="project" select=".."/>
					<xsl:with-param name="target_name" select="@name"/>
				</xsl:call-template>
			</pre>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$ant_runtime">
				<table>
					<tr>
						<td>
							<xsl:call-template name="_print_ant_runtime_a">
								<xsl:with-param name="target_name" select="@name"/>
							</xsl:call-template>
						</td>
						<td>
							<xsl:copy-of select="$task_header"/>
						</td>
					</tr>
				</table>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="$task_header"/>
			</xsl:otherwise>
		</xsl:choose>
		<hr/>
		<dl>
			<dt>
				<xsl:call-template name="_print_summary_target_description"/>
			</dt>
		</dl>
		<hr/>
		<dl>
			<dt>
				<b>Detail:</b>
				<table>
					<tr class="TableSubHeadingColor" bgcolor="#EEEEFF">
						<td>
							<pre>
								<xsl:apply-templates select="." mode="htmlout"/>
							</pre>
						</td>
					</tr>
				</table>
			</dt>
		</dl>
	</xsl:template>
	<!--
	Generates the HTML documentation for a single target.
	The current node must be the "project" node, and the "single_target_name" parameter must be equal to the name of the target to be documented.
	-->
	<xsl:template name="print_documentation_for_a_single_target">
		<html>
			<head>
				<xsl:call-template name="_print_link_and_meta_head"/>
				<title>
					<xsl:value-of select="@name"/> AntDoc</title>
				<script language="javascript">
function run_ant_target(target_name)
{
	if (document.<xsl:value-of select="$antdoc_applet_name"/> != null)
	{
		document.<xsl:value-of select="$antdoc_applet_name"/>.setTarget(target_name);
	}
}
</script>
			</head>
			<body bgcolor="white">
				<xsl:for-each select="target[@name = $single_target_name]">
					<xsl:call-template name="_print_full_target_details"/>
				</xsl:for-each>
			</body>
		</html>
	</xsl:template>
	<!--
	Generates the HTML documentation for the description of an Ant buildfile project.
	The current node must be the "project" node to be documented.
	-->
	<xsl:template name="print_documentation_for_a_project_description">
		<html>
			<head>
				<xsl:call-template name="_print_link_and_meta_head"/>
				<title>
					<xsl:value-of select="@name"/> AntDoc</title>
				<script language="javascript">
function run_ant_target(target_name)
{
	if (document.<xsl:value-of select="$antdoc_applet_name"/> != null)
	{
		document.<xsl:value-of select="$antdoc_applet_name"/>.setTarget(target_name);
	}
}
</script>
			</head>
			<body bgcolor="white">
				<xsl:call-template name="_print_project_description">
					<xsl:with-param name="ant_script_filename" select="$xml_source_file_url"/>
				</xsl:call-template>
			</body>
		</html>
	</xsl:template>
	<!--Prints the copyright footer display at the bottom of a page, after the navigation bottom-->
	<xsl:template name="_print_copyright_footer">
		<hr/>
		<div align="right">
			<i>
				<a href="mailto:{$tool_author_email}">
					<xsl:value-of select="$tool_author"/>
				</a>
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$tool_lastupdate"/>
				<xsl:text>&#160;-&#160;</xsl:text>
				<xsl:value-of select="$tool_name"/>
				<xsl:text>&#160;V</xsl:text>
				<xsl:value-of select="$tool_version"/>
				<xsl:text>&#160;-&#160;XSL Transformer Factory: </xsl:text>
				<code>
					<xsl:value-of select="$transformer_factory_class_fqn"/>
				</code>
			</i>
		</div>
	</xsl:template>
	<!--Prints an Ant runtime icon so that the underlying target may be executed-->
	<!--the target_name parameter should be provided, and should be equal to the Ant target name to be executed-->
	<xsl:template name="_print_ant_runtime_a">
		<xsl:param name="target_name"/>
		<a>
			<xsl:attribute name="href">javascript:run_ant_target('<xsl:value-of select="@name"/>');</xsl:attribute>
			<img alt="Press that icon in order to update AntDoc GUI" src="{$antdoc_gui_logo_image_file_name}"/>
		</a>
	</xsl:template>
	<!--
	Prints the file name of the generated HTML page for the given Ant buildfile.
	The 'ant_script_file' parameter should be provided, and should be equal to the full path of the Ant script.
	The 'project_name' optional parameter may be provided; in that case, it should be equal to the Ant buildfile project name, and the previous parameter is ignored.
	-->
	<xsl:template name="_print_project_file_name">
		<xsl:param name="ant_script_file"/>
		<xsl:param name="project_name"/>
		<xsl:choose>
			<xsl:when test="$project_name">
				<xsl:value-of select="$project_name"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="document($ant_script_file)/project/@name"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:text>.html</xsl:text>
	</xsl:template>
	<!--
	Prints the relative (to the destination directory) URL of the generated HTML page for the given Ant buildfile.
	The 'project_name' is expected and should be equal to the name of the project.
	-->
	<xsl:template name="_print_project_href">
		<xsl:param name="project_name"/>
		<xsl:call-template name="_print_html_root_href">
			<xsl:with-param name="project_name" select="$project_name"/>
		</xsl:call-template>
		<xsl:call-template name="_print_project_file_name">
			<xsl:with-param name="project_name" select="$project_name"/>
		</xsl:call-template>
	</xsl:template>
	<!--
	Prints the directory path of the destination directory where HTML pages are output.
	Warning, this path ends with a '/'!
	If the 'project_name' parameter is provided, the output directory path returned concerns the specific Ant project defined by its name.
	-->
	<xsl:template name="_print_directory_output_path">
		<xsl:param name="project_name"/>
		<xsl:value-of select="$output_path"/>
		<xsl:text>./</xsl:text>
	</xsl:template>
	<!--
	Prints the root URL for all HTML pages.
	Warning, this URL with end with a '/' if necessary!
	If the 'project_name' parameter is provided, the URL returned concerns the specific Ant project defined by its name.
	-->
	<xsl:template name="_print_html_root_href">
		<xsl:param name="project_name"/>
	</xsl:template>
	<!--
	Prints the file name of the generated HTML page for the given Ant target.
	The 'target_name' parameter is expected and should be equal to the name of the Ant target.
	The 'project_name' parameter is expected and should be equal to the name of the Ant project in which the target is defined.
	-->
	<xsl:template name="_print_target_html_file_name">
		<xsl:param name="target_name"/>
		<xsl:param name="project_name"/>
		<xsl:call-template name="_print_directory_output_path">
			<xsl:with-param name="project_name" select="$project_name"/>
		</xsl:call-template>
		<xsl:text>/</xsl:text>
		<xsl:value-of select="../@name"/>-<xsl:value-of select="@name"/>
		<xsl:text>.html</xsl:text>
	</xsl:template>
	<!--
	Prints the URL of the generated HTML page for the given Ant target.
	The 'target_name' parameter is expected and should be equal to the name of the Ant target.
	The 'project_name' parameter is expected and should be equal to the name of the Ant project in which the target is defined.
	-->
	<xsl:template name="_print_target_href">
		<xsl:param name="target_name"/>
		<xsl:param name="project_name"/>
		<xsl:call-template name="_print_html_root_href">
			<xsl:with-param name="project_name" select="$project_name"/>
		</xsl:call-template>
		<xsl:value-of select="$project_name"/>-<xsl:value-of select="$target_name"/>
		<xsl:text>.html</xsl:text>
	</xsl:template>
	<!--
	Prints the file name of the generated HTML targets overview for the given Ant buidfile file.
	The 'ant_script_file' parameter should be provided, and should be equal to the full path of the Ant buildfile.
	-->
	<xsl:template name="_print_ant_target_overview_html_file_name">
		<xsl:param name="ant_script_file"/>
		<xsl:value-of select="document($ant_script_file)/project/@name"/>
		<xsl:text>-overview.html</xsl:text>
	</xsl:template>
	<!--
	Prints the relative (to the destination directory) URL of the generated HTML targets overview for the given Ant script file.
	The 'ant_script_file' parameter should be provided, and should be equal to the full path of the Ant script.
	-->
	<xsl:template name="_print_ant_target_overview_href">
		<xsl:param name="context"/>
		<xsl:param name="ant_script_file"/>
		<xsl:variable name="project_name">
			<xsl:value-of select="document($ant_script_file)/project/@name"/>
		</xsl:variable>
		<xsl:call-template name="_print_html_root_href">
			<xsl:with-param name="project_name" select="$project_name"/>
		</xsl:call-template>
		<xsl:call-template name="_print_ant_target_overview_html_file_name">
			<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
		</xsl:call-template>
	</xsl:template>
	<!--
	Prints the full static target dependencies, i.e. the target dependency tree of a given target.
	The 'project' parameter should be provided and equal to the Ant 'project' XML element where the target lies.
	The 'target_name' parameter must be provided, and be equal to the target name for which the dependency tree is being printed.
	The current node must belong to the Ant buildfile XML document that contains the given target.
	-->
	<xsl:template name="_print_static_target_dependencies">
		<xsl:param name="project"/>
		<xsl:param name="target_name"/>
		<b>
			<xsl:call-template name="_print_project_target_a">
				<xsl:with-param name="project" select="$project"/>
				<xsl:with-param name="target_name" select="$target_name"/>
			</xsl:call-template>
		</b>
		<xsl:for-each select="/project/AntDocDependency/AntDocDependency[@type = 'static' and @name = $target_name]/AntDocDependency">
			<xsl:call-template name="_print_static_target_dependencies_sub">
				<xsl:with-param name="project" select="$project"/>
				<xsl:with-param name="indentation" select="'&#160;&#160;'"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	<!--
	This is a helper template for the _print_static_target_dependencies, that enables to output an Ant target dependency tree.
	The current node must be an AntDocDependency element.
	The 'project' parameter is expected and should be equal to the Ant 'project' XML element that contains the target with name '@name'.
	-->
	<xsl:template name="_print_static_target_dependencies_sub">
		<xsl:param name="project"/>
		<xsl:param name="indentation"/>
		<br/>
		<xsl:value-of select="$indentation"/>
		<xsl:text>|</xsl:text>
		<br/>
		<xsl:value-of select="$indentation"/>
		<xsl:text>+--</xsl:text>
		<xsl:call-template name="_print_project_target_a">
			<xsl:with-param name="project" select="$project"/>
			<xsl:with-param name="target_name" select="@name"/>
		</xsl:call-template>
		<xsl:variable name="child_indentation">
			<xsl:text>&#160;&#160;</xsl:text>
			<xsl:value-of select="$indentation"/>
		</xsl:variable>
		<xsl:for-each select="AntDocDependency">
			<xsl:call-template name="_print_static_target_dependencies_sub">
				<xsl:with-param name="project" select="$project"/>
				<xsl:with-param name="indentation" select="$child_indentation"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	<!--
	Prints the full runtime target dependencies, i.e. all target that will be called at runtime before the provided target.
	The 'project' parameter is expected and must be equal to the Ant 'project' XML element that contains the following Ant target.
	The 'target_name' parameter must be provided, and be equal to the target name for which this runtime execution path is being searched for.
	-->
	<xsl:template name="_print_runtime_target_dependencies">
		<xsl:param name="project"/>
		<xsl:param name="target_name"/>
		<xsl:for-each select="/project/AntDocDependency/AntDocDependency[@type = 'runtime' and @name = $target_name]/AntDocDependency">
			<xsl:call-template name="_print_project_target_a">
				<xsl:with-param name="project" select="$project"/>
				<xsl:with-param name="target_name" select="@name"/>
			</xsl:call-template>
			<xsl:if test="position() != last()">
				<br/>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<!--Prints a list of Token elements from a string corresponding to the token separated by comma in the original string-->
	<!--the full_string parameter should be provided-->
	<xsl:template name="_print_tokens">
		<xsl:param name="full_string"/>
		<xsl:variable name="token">
			<xsl:choose>
				<xsl:when test="string-length(substring-before($full_string, ',')) > 0">
					<xsl:value-of select="normalize-space(substring-before($full_string, ','))"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="normalize-space($full_string)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="string-length($token) > 0">
			<Token>
				<xsl:value-of select="$token"/>
			</Token>
		</xsl:if>
		<xsl:variable name="string_left" select="substring-after($full_string, ',')"/>
		<xsl:if test="string-length($string_left) > 0">
			<xsl:text>&#032;</xsl:text>
			<xsl:call-template name="_print_tokens">
				<xsl:with-param name="full_string" select="$string_left"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	<!--
	Prints the header and footer for all pages generated for the right frame.
	The 'text' parameter is expected and is the message that will be used in order to identify the page.
	The 'location' parameter is the location of the page in which the index should be written: possible values are: {overview, script, target, usage, about}.
	The 'project' parameter may be provided and if yes, it must be the Ant project top node corresponding to the Ant target for which generation is performed. It is mandatory if the 'location' parameter is equal to 'target'.
	-->
	<xsl:template name="_print_navigation">
		<xsl:param name="text"/>
		<xsl:param name="location"/>
		<xsl:param name="project"/>
		<table border="0" cellpadding="1" cellspacing="0" width="100%">
			<tbody>
				<tr>
					<td class="NavBarCell1" bgcolor="#EEEEFF">
						<table cellspacing="3" cellpadding="0" border="0">
							<tbody>
								<tr valign="top" align="middle">
									<td>
										<xsl:call-template name="_print_navigation_top_td_class_and_bgcolor">
											<xsl:with-param name="location" select="$location"/>
											<xsl:with-param name="compared_location" select="'overview'"/>
										</xsl:call-template>
										<xsl:text>&#160;</xsl:text>
										<xsl:variable name="overview_tree">
											<font>
												<xsl:call-template name="_print_navigation_top_td_font_class">
													<xsl:with-param name="location" select="$location"/>
													<xsl:with-param name="compared_location" select="'overview'"/>
												</xsl:call-template>
												<b>Overview</b>
											</font>
										</xsl:variable>
										<xsl:choose>
											<xsl:when test="$location = 'overview'">
												<xsl:copy-of select="$overview_tree"/>
											</xsl:when>
											<xsl:otherwise>
												<a href="{$welcome_file}" target="{$content_target}">
													<xsl:copy-of select="$overview_tree"/>
												</a>
											</xsl:otherwise>
										</xsl:choose>
										<xsl:text>&#160;</xsl:text>
									</td>
									<td>
										<xsl:call-template name="_print_navigation_top_td_class_and_bgcolor">
											<xsl:with-param name="location" select="$location"/>
											<xsl:with-param name="compared_location" select="'script'"/>
										</xsl:call-template>
										<xsl:text>&#160;</xsl:text>
										<xsl:variable name="script_tree">
											<font>
												<xsl:call-template name="_print_navigation_top_td_font_class">
													<xsl:with-param name="location" select="$location"/>
													<xsl:with-param name="compared_location" select="'script'"/>
												</xsl:call-template>Project</font>
										</xsl:variable>
										<xsl:choose>
											<xsl:when test="$location = 'script'">
												<b>
													<xsl:copy-of select="$script_tree"/>
												</b>
											</xsl:when>
											<xsl:otherwise>
												<xsl:choose>
													<xsl:when test="$location = 'target'">
														<b>
															<a>
																<xsl:attribute name="href"><xsl:call-template name="_print_project_href"><xsl:with-param name="project_name" select="$project/@name"/></xsl:call-template></xsl:attribute>
																<xsl:copy-of select="$script_tree"/>
															</a>
														</b>
													</xsl:when>
													<xsl:otherwise>
														<xsl:copy-of select="$script_tree"/>
													</xsl:otherwise>
												</xsl:choose>
											</xsl:otherwise>
										</xsl:choose>
										<xsl:text>&#160;</xsl:text>
									</td>
									<td>
										<xsl:call-template name="_print_navigation_top_td_class_and_bgcolor">
											<xsl:with-param name="location" select="$location"/>
											<xsl:with-param name="compared_location" select="'target'"/>
										</xsl:call-template>
										<xsl:text>&#160;</xsl:text>
										<xsl:variable name="target_tree">
											<font>
												<xsl:call-template name="_print_navigation_top_td_font_class">
													<xsl:with-param name="location" select="$location"/>
													<xsl:with-param name="compared_location" select="'target'"/>
												</xsl:call-template>Target</font>
										</xsl:variable>
										<xsl:choose>
											<xsl:when test="$location = 'target'">
												<b>
													<xsl:copy-of select="$target_tree"/>
												</b>
											</xsl:when>
											<xsl:otherwise>
												<xsl:copy-of select="$target_tree"/>
											</xsl:otherwise>
										</xsl:choose>
										<xsl:text>&#160;</xsl:text>
									</td>
									<td>
										<xsl:call-template name="_print_navigation_top_td_class_and_bgcolor">
											<xsl:with-param name="location" select="$location"/>
											<xsl:with-param name="compared_location" select="'usage'"/>
										</xsl:call-template>
										<xsl:text>&#160;</xsl:text>
										<xsl:variable name="usage_tree">
											<font>
												<xsl:call-template name="_print_navigation_top_td_font_class">
													<xsl:with-param name="location" select="$location"/>
													<xsl:with-param name="compared_location" select="'usage'"/>
												</xsl:call-template>
												<b>Usage</b>
											</font>
										</xsl:variable>
										<xsl:choose>
											<xsl:when test="$location = 'usage'">
												<xsl:copy-of select="$usage_tree"/>
											</xsl:when>
											<xsl:otherwise>
												<a href="{$task_usage_file}" target="{$content_target}">
													<xsl:copy-of select="$usage_tree"/>
												</a>
											</xsl:otherwise>
										</xsl:choose>
										<xsl:text>&#160;</xsl:text>
									</td>
									<td>
										<xsl:call-template name="_print_navigation_top_td_class_and_bgcolor">
											<xsl:with-param name="location" select="$location"/>
											<xsl:with-param name="compared_location">about</xsl:with-param>
										</xsl:call-template>
										<xsl:text>&#160;</xsl:text>
										<xsl:variable name="about_tree">
											<font>
												<xsl:call-template name="_print_navigation_top_td_font_class">
													<xsl:with-param name="location" select="$location"/>
													<xsl:with-param name="compared_location">about</xsl:with-param>
												</xsl:call-template>
												<b>About</b>
											</font>
										</xsl:variable>
										<xsl:choose>
											<xsl:when test="$location = 'about'">
												<xsl:copy-of select="$about_tree"/>
											</xsl:when>
											<xsl:otherwise>
												<a href="{$about_file}" target="{$content_target}">
													<xsl:copy-of select="$about_tree"/>
												</a>
											</xsl:otherwise>
										</xsl:choose>
										<xsl:text>&#160;</xsl:text>
									</td>
								</tr>
							</tbody>
						</table>
					</td>
					<td valign="top" align="right" rowspan="2">
						<em>
							<xsl:value-of select="$text" disable-output-escaping="yes"/>
						</em>
					</td>
				</tr>
				<tr>
					<xsl:if test="$location = 'script'">
						<td class="NavBarCell3" valign="top">
							<font size="-2">
								<xsl:text>INDEX:&#160;</xsl:text>
								<a href="#summary">
									<xsl:text>SUMMARY</xsl:text>
								</a>
								<xsl:text>&#160;|&#160;</xsl:text>
								<a href="#property">
									<xsl:text>PROPERTY</xsl:text>
								</a>
								<xsl:text>&#160;|&#160;</xsl:text>
								<a href="#detail">
									<xsl:text>DETAIL</xsl:text>
								</a>
							</font>
						</td>
					</xsl:if>
				</tr>
			</tbody>
		</table>
	</xsl:template>
	<xsl:template name="_print_navigation_top_td_class_and_bgcolor">
		<!--Prints the index top td class and background color attributes-->
		<!--the location where the attribute is generated-->
		<xsl:param name="location"/>
		<!--the location the attribute is compared to-->
		<xsl:param name="compared_location"/>
		<xsl:variable name="top_td_class_unselected">NavBarCell1</xsl:variable>
		<xsl:variable name="top_td_class_selected">NavBarCell1Rev</xsl:variable>
		<xsl:variable name="top_td_bgcolor_unselected">#EEEEFF</xsl:variable>
		<xsl:variable name="top_td_bgcolor_selected">#FFFFFF</xsl:variable>
		<xsl:attribute name="class"><xsl:choose><xsl:when test="$location = $compared_location"><xsl:value-of select="$top_td_class_selected"/></xsl:when><xsl:otherwise><xsl:value-of select="$top_td_class_unselected"/></xsl:otherwise></xsl:choose></xsl:attribute>
		<xsl:attribute name="bgcolor"><xsl:choose><xsl:when test="$location = $compared_location"><xsl:value-of select="$top_td_bgcolor_selected"/></xsl:when><xsl:otherwise><xsl:value-of select="$top_td_bgcolor_unselected"/></xsl:otherwise></xsl:choose></xsl:attribute>
	</xsl:template>
	<!--
		Prints the index top td font class attribute.
		The 'location' is where the attribute is generated.
		The 'compared_location' is the attribute is compared to.
		-->
	<xsl:template name="_print_navigation_top_td_font_class">
		<xsl:param name="location"/>
		<xsl:param name="compared_location"/>
		<xsl:variable name="top_td_font_class_unselected">NavBarFont1</xsl:variable>
		<xsl:variable name="top_td_font_class_selected">NavBarFont1Rev</xsl:variable>
		<xsl:attribute name="class"><xsl:choose><xsl:when test="$location = $compared_location"><xsl:value-of select="$top_td_font_class_selected"/></xsl:when><xsl:otherwise><xsl:value-of select="$top_td_font_class_unselected"/></xsl:otherwise></xsl:choose></xsl:attribute>
	</xsl:template>
	<!--The template tests whether the given parameter name is an Ant task, and prints the task accordingly-->
	<!--The name parameter should be given as an argument and equal to the name of the task-->
	<xsl:template name="_print_ant_task">
		<xsl:param name="name"/>
		<xsl:choose>
			<xsl:when test="count(document($ant_tasks_file)//AntTask[@name =$name]) > 0">
				<a>
					<xsl:attribute name="href"><xsl:value-of select="$ant_url"/><xsl:text>/</xsl:text><xsl:value-of select="document($ant_tasks_file)//AntTask[@name =$name]"/></xsl:attribute>
					<xsl:value-of select="$name"/>
				</a>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$name"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
	Prints the standard AntDoc link and meta code that should be present within the 'head' HTML element.
	If the 'relative_path' parameter is defined, it corresponds to the gap of path between the current XML file output and the destination directory. This path must not end with a '/'. It is being used in order to ouput a 'base' element.
	-->
	<xsl:template name="_print_link_and_meta_head">
		<xsl:param name="relative_path"/>
		<xsl:if test="$relative_path">		</xsl:if>
		<meta name="Author">
			<xsl:attribute name="content"><xsl:value-of select="$tool_author"/></xsl:attribute>
		</meta>
		<meta name="Copyright">
			<xsl:attribute name="content"><xsl:value-of select="$tool_author"/>, <xsl:value-of select="$tool_lastupdate"/></xsl:attribute>
		</meta>
		<xsl:variable name="stylesheet_href">
			<!--xsl:if test="$relative_path">
				<xsl:value-of select="$relative_path"/>
				<xsl:text>/</xsl:text>
			</xsl:if-->
			<xsl:text>stylesheet.css</xsl:text>
		</xsl:variable>
		<link rel="StyleSheet" type="text/css">
			<xsl:attribute name="href"><xsl:value-of select="$stylesheet_href"/></xsl:attribute>
		</link>
	</xsl:template>
	<!--
	Prints an HTML reference to a target in a given project. Plain text is output.
	The 'ant_script_file' parameter should be provided, and should be equal to the full path of the Ant script.
	The 'target_name' parameter should be provided and equal to the desired target name: it must be a string type, not an attribute.
	-->
	<xsl:template name="_print_project_target_href">
		<xsl:param name="ant_script_file"/>
		<xsl:param name="target_name"/>
		<xsl:call-template name="_print_target_href">
			<xsl:with-param name="target_name" select="$target_name"/>
			<xsl:with-param name="project_name" select="document($ant_script_file)/project/@name"/>
		</xsl:call-template>
	</xsl:template>
	<!--
	Prints the hyperlink to a target not necessarily in the current project: the output is an anchor with a text value depending on the parameters.
	The 'project' parameter should be provided and equal to the Ant 'project' XML element where the target lies.
	The 'target_name' parameter may be provided and equal to the desired target name: it must be a string type, not an attribute.
	-->
	<xsl:template name="_print_project_target_a">
		<xsl:param name="project"/>
		<xsl:param name="target_name"/>
		<xsl:variable name="project_name" select="$project/@name"/>
		<xsl:choose>
			<xsl:when test="count($project/target[@name = $target_name]) > 0">
				<a>
					<xsl:attribute name="href"><xsl:call-template name="_print_target_href"><xsl:with-param name="project_name" select="$project_name"/><xsl:with-param name="target_name" select="$target_name"/></xsl:call-template></xsl:attribute>
					<xsl:value-of select="$target_name"/>
				</a>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$target_name"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--
	Prints an hyperlink to an Ant project. The hyperlink makes two frames update at the same time.
	The 'doc_file' parameter is expected and should be equal to the URL of the Ant buildfile HTML page of the project.
	The 'ant_script_file' parameter is expected and should be equal to the path of the Ant buildfile corresponding to the project.
	The 'ant_project_name' parameter is expected and should be equal to the name of the project.
	-->
	<xsl:template name="_print_project_a">
		<xsl:param name="context"/>
		<xsl:param name="doc_file"/>
		<xsl:param name="ant_script_file"/>
		<xsl:param name="ant_project_name"/>
		<xsl:call-template name="_print_a_from_top_left_frame">
			<xsl:with-param name="right_url" select="$doc_file"/>
			<xsl:with-param name="left_bottom_url">
				<xsl:call-template name="_print_ant_target_overview_href">
					<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
				</xsl:call-template>
			</xsl:with-param>
			<xsl:with-param name="text" select="$ant_project_name"/>
		</xsl:call-template>
	</xsl:template>
	<!--
	Prints an hyperlink that should be printed in the left top frame.
	The 'right_url' parameter is expected and should be equal to the URL of the HTML page that will be displayed on the right frame by clicking the hyperlink.
	The 'left_bottom_url' parameter is expected and should be equal to the URL of the HTML page that will be displayed on the left bottom frame by clicking the hyperlink.
	The 'text' parameter is expected and should be equal to the hyperlink text.
	-->
	<xsl:template name="_print_a_from_top_left_frame">
		<xsl:param name="right_url"/>
		<xsl:param name="left_bottom_url"/>
		<xsl:param name="text"/>
		<a href="{$right_url}" target="{$content_target}">
			<xsl:attribute name="onclick">parent.<xsl:value-of select="$targets_target"/>.location='<xsl:value-of select="$left_bottom_url"/>'</xsl:attribute>
			<xsl:value-of select="$text"/>
		</a>
	</xsl:template>
	<!--
	Prints the description of the target.
	The current node should be the target element for which the description should be output.
	-->
	<xsl:template name="_print_summary_target_description">
		<span id="{$antdoc_css_class_name}">
			<xsl:if test="@description">
				<p>
					<xsl:value-of select="@description" disable-output-escaping="yes"/>
				</p>
			</xsl:if>
			<xsl:if test="preceding-sibling::node()[normalize-space()][1][self::comment()]">
				<font color="{$xml_comments_color}">
					<xsl:value-of select="preceding-sibling::node()[normalize-space()][1][self::comment()]" disable-output-escaping="yes"/>
				</font>
			</xsl:if>
		</span>
	</xsl:template>
	<!--
	Writes the HTML index file.
	-->
	<xsl:template name="write_index">
		<xsl:variable name="file">
			<xsl:call-template name="_print_directory_output_path"/>
			<xsl:text>index.html</xsl:text>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="$title"/>
					</title>
				</head>
				<frameset cols="20%,80%">
					<frameset rows="30%,70%">
						<frame src="{$menu_file}"/>
						<frame name="{$targets_target}" src="{$target_overview_file}"/>
					</frameset>
					<frame name="{$content_target}" src="{$welcome_file}"/>
				</frameset>
				<noframes>
					<h2>Frame Alert</h2>
					<p>This document is designed to be viewed using the frames feature. If you see this message, you are using a non-frame-capable web client.</p>
				</noframes>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Writes the HTML menu file, which is located in the top left frame.
	-->
	<xsl:template name="write_menu">
		<xsl:variable name="file">
			<xsl:value-of select="$output_path"/>
			<xsl:text>/</xsl:text>
			<xsl:value-of select="$menu_file"/>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="$title"/>
						<xsl:text> Ant buildfiles</xsl:text>
					</title>
				</head>
				<body bgcolor="white">
					<table width="100%">
						<tbody>
							<tr>
								<td>
									<font class="FrameTitleFont" size="+1">
										<b>
											<xsl:value-of select="$title" disable-output-escaping="yes"/>
										</b>
									</font>
								</td>
							</tr>
						</tbody>
					</table>
					<table width="100%" border="0">
						<tbody>
							<tr>
								<td nowrap="nowrap">
									<xsl:if test="count(Script) > 1">
										<font class="FrameItemFont">
											<xsl:call-template name="_print_a_from_top_left_frame">
												<xsl:with-param name="right_url" select="$welcome_file"/>
												<xsl:with-param name="left_bottom_url" select="$target_overview_file"/>
												<xsl:with-param name="text" select="'All Targets'"/>
											</xsl:call-template>
										</font>
									</xsl:if>
									<p>
										<font size="+1" class="FrameHeadingFont">Projects</font>
										<br/>
										<xsl:for-each select="Script">
											<xsl:sort order="ascending" select="."/>
											<xsl:variable name="ant_script_file" select="string(@filtered)"/>
											<xsl:variable name="ant_project_name" select="document($ant_script_file)/project/@name"/>
											<xsl:variable name="doc_file">
												<xsl:call-template name="_print_project_href">
													<xsl:with-param name="project_name" select="$ant_project_name"/>
												</xsl:call-template>
											</xsl:variable>
											<font class="FrameHeadingFont">
												<xsl:call-template name="_print_project_a">
													<xsl:with-param name="doc_file" select="$doc_file"/>
													<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
													<xsl:with-param name="ant_project_name" select="$ant_project_name"/>
												</xsl:call-template>
											</font>
											<br/>
										</xsl:for-each>
									</p>
								</td>
							</tr>
						</tbody>
					</table>
				</body>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Writes the welcome HTML file, which is first displayed in the left pane.
	The current node must be the root node ('Scripts') of the XML file containing all Ant buildfiles to be documented.
	-->
	<xsl:template name="write_welcome">
		<xsl:variable name="file">
			<xsl:value-of select="$output_path"/>
			<xsl:value-of select="$welcome_file"/>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="$tool_name"/>
						<xsl:text> V</xsl:text>
						<xsl:value-of select="$tool_version"/>
						<xsl:text> welcome page</xsl:text>
					</title>
				</head>
				<body bgcolor="white">
					<xsl:call-template name="_print_navigation">
						<xsl:with-param name="text" select="$header_text"/>
						<xsl:with-param name="location" select="'overview'"/>
					</xsl:call-template>
					<hr/>
					<center>
						<h2>
							<xsl:value-of select="$title" disable-output-escaping="yes"/>
						</h2>
					</center>
					<br/>
					<br/>
					<xsl:choose>
						<xsl:when test="$welcome_html_content">
							<xsl:value-of select="document($welcome_html_content)/body" disable-output-escaping="yes"/>
						</xsl:when>
						<xsl:otherwise>
							<br/>
							<br/>
							<br/>
							<font size="+1">
								<p>
									<i>You may use an HTML welcome page by using the "<code>welcome</code>" attribute.</i>
								</p>
							</font>
						</xsl:otherwise>
					</xsl:choose>
					<table border="1" cellpadding="3" cellspacing="0" width="100%">
						<tr class="TableHeadingColor" bgcolor="#CCCCFF">
							<th colspan="2">
								<font size="+2">
									<b>Projects</b>
								</font>
							</th>
						</tr>
						<xsl:for-each select="Script">
							<xsl:variable name="ant_script_file" select="string(@filtered)"/>
							<xsl:variable name="ant_project_name" select="document($ant_script_file)/project/@name"/>
							<xsl:variable name="doc_file">
								<xsl:call-template name="_print_project_href">
									<xsl:with-param name="project_name" select="$ant_project_name"/>
								</xsl:call-template>
							</xsl:variable>
							<tr>
								<td>
									<b>
										<xsl:call-template name="_print_project_a">
											<xsl:with-param name="doc_file" select="$doc_file"/>
											<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
											<xsl:with-param name="ant_project_name" select="$ant_project_name"/>
										</xsl:call-template>
									</b>
								</td>
								<td>
									<xsl:value-of select="document($ant_script_file)/project/description" disable-output-escaping="yes"/>
								</td>
							</tr>
						</xsl:for-each>
					</table>
				</body>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Writes the HTML file that displays the full detail concerning the given Ant target, which should be the current node.
	-->
	<xsl:template name="write_target">
		<xsl:variable name="file">
			<xsl:call-template name="_print_target_html_file_name">
				<xsl:with-param name="target_name" select="@name"/>
				<xsl:with-param name="project_name" select="../@name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="@name"/>
						<xsl:text> target page</xsl:text>
					</title>
				</head>
				<body bgcolor="white">
					<xsl:call-template name="_print_navigation">
						<xsl:with-param name="text" select="$header_text"/>
						<xsl:with-param name="location" select="'target'"/>
						<xsl:with-param name="project" select=".."/>
					</xsl:call-template>
					<hr/>
					<xsl:call-template name="_print_full_target_details"/>
					<hr/>
					<xsl:call-template name="_print_navigation">
						<xsl:with-param name="text" select="$footer_text"/>
						<xsl:with-param name="location" select="'target'"/>
						<xsl:with-param name="project" select=".."/>
					</xsl:call-template>
					<xsl:call-template name="_print_copyright_footer"/>
				</body>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Writes the HTML file, which displays all task uses. This page indicates where each Ant task is being used.
	-->
	<xsl:template name="write_task_usage">
		<xsl:variable name="file">
			<xsl:value-of select="$output_path"/>
			<xsl:text>/</xsl:text>
			<xsl:value-of select="$task_usage_file"/>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="$tool_name"/>
						<xsl:text> V</xsl:text>
						<xsl:value-of select="$tool_version"/>
						<xsl:text> task usage page</xsl:text>
					</title>
				</head>
				<body bgcolor="white">
					<xsl:call-template name="_print_navigation">
						<xsl:with-param name="text" select="$header_text"/>
						<xsl:with-param name="location" select="'usage'"/>
					</xsl:call-template>
					<hr/>
					<center>
						<h2>Ant task usage</h2>
					</center>
					<br/>
					<br/>
					<font size="+1">
						<i>This page displays all Ant tasks being used among the generated Ant buidfile(s) documentation. You should be able to browse their documentation, but also to see the underlying usage code.</i>
					</font>
					<br/>
					<br/>
					<table border="1" cellpadding="3" cellspacing="0" width="100%">
						<tr class="TableHeadingColor" bgcolor="#CCCCFF">
							<td colspan="2">
								<font size="+2">
									<b>Task Usage</b>
								</font>
							</td>
						</tr>
						<xsl:variable name="ant_task_location">
							<tasks>
								<xsl:for-each select="Script">
									<xsl:variable name="ant_script_file" select="string(@filtered)"/>
									<xsl:variable name="ant_project_name" select="document($ant_script_file)/project/@name"/>
									<xsl:variable name="ant_script_local_file" select="@local"/>
									<xsl:variable name="ant_script_local_javascript_file" select="@local_javascript"/>
									<xsl:for-each select="document($ant_script_file)/project/target/*">
										<task>
											<xsl:attribute name="name"><xsl:value-of select="name()"/></xsl:attribute>
											<xsl:attribute name="buildfile"><xsl:value-of select="$ant_script_file"/></xsl:attribute>
											<xsl:attribute name="project"><xsl:value-of select="$ant_project_name"/></xsl:attribute>
											<xsl:attribute name="target"><xsl:value-of select="parent::target/@name"/></xsl:attribute>
										</task>
									</xsl:for-each>
								</xsl:for-each>
							</tasks>
						</xsl:variable>
						<xsl:for-each select="$ant_task_location/tasks/task[generate-id() = generate-id(key('task_key', @name))]">
							<xsl:sort select="@name"/>
							<tr class="TableRowColor" bgcolor="white">
								<td align="right" valign="top" width="1%">
									<b>
										<code>
											<xsl:call-template name="_print_ant_task">
												<xsl:with-param name="name" select="string(@name)"/>
											</xsl:call-template>
										</code>
									</b>
								</td>
								<td>
									<xsl:for-each select="../task[@name = current()/@name]">
										<xsl:sort select="@project"/>
										<xsl:sort select="@target"/>
										<code>
											<font size="+1">
												<xsl:value-of select="@project"/>
											</font>.<a>
												<xsl:attribute name="href"><xsl:call-template name="_print_project_target_href"><xsl:with-param name="ant_script_file" select="string(@buildfile)"/><xsl:with-param name="target_name" select="string(@target)"/></xsl:call-template></xsl:attribute>
												<xsl:value-of select="@target"/>
											</a>
										</code>
										<xsl:if test="not(position() = last())">
											<xsl:text>, </xsl:text>
										</xsl:if>
									</xsl:for-each>
								</td>
							</tr>
						</xsl:for-each>
					</table>
					<br/>
					<hr/>
					<xsl:call-template name="_print_navigation">
						<xsl:with-param name="text" select="$footer_text"/>
						<xsl:with-param name="location" select="'usage'"/>
					</xsl:call-template>
					<xsl:call-template name="_print_copyright_footer"/>
				</body>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Writes the about HTML file.
	-->
	<xsl:template name="write_about">
		<xsl:variable name="file">
			<xsl:value-of select="$output_path"/>
			<xsl:text>/</xsl:text>
			<xsl:value-of select="$about_file"/>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="$tool_name"/>
						<xsl:text> V</xsl:text>
						<xsl:value-of select="$tool_version"/>
						<xsl:text> about page</xsl:text>
					</title>
				</head>
				<body bgcolor="white">
					<xsl:call-template name="_print_navigation">
						<xsl:with-param name="text" select="$header_text"/>
						<xsl:with-param name="location" select="'about'"/>
					</xsl:call-template>
					<hr/>
					<center>
						<h2>
							<xsl:value-of select="$tool_name"/>
							<xsl:text> about page</xsl:text>
						</h2>
					</center>
					<br/>
					<br/>
					<br/>
					<font size="+1">
						<i>
							<p>
								<xsl:text>You may download the latest version of that tool by </xsl:text>
								<a href="{$tool_author_web_site}">clicking here</a>.
						</p>
							<p>
								<xsl:text>AntDoc has now a GUI, which is named AntDocGUI: </xsl:text>
								<a href="{$tool_author_web_site}/AntDocGUI%20Documentation_content.php"> take a look</a>.
						</p>
							<p>Go to <a href="{$ant_url}">Ant official web site</a>. Its <a href="{$ant_url}/manual"> manual</a>.</p>
						</i>
					</font>
				</body>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Writes the targets overview HTML file, which is first displayed in the bottom left pane.
	-->
	<xsl:template name="write_overview">
		<xsl:variable name="file">
			<xsl:value-of select="$output_path"/>
			<xsl:text>/</xsl:text>
			<xsl:value-of select="$target_overview_file"/>
		</xsl:variable>
		<xsl:result-document href="{$file}">
			<html>
				<head>
					<xsl:call-template name="_print_link_and_meta_head"/>
					<title>
						<xsl:value-of select="$title"/>
						<xsl:text> Ant targets overview</xsl:text>
					</title>
				</head>
				<body bgcolor="white">
					<font size="+1" class="FrameHeadingFont">
						<b>All Targets</b>
					</font>
					<br/>
					<table border="0" width="100%">
						<tr>
							<td>
								<xsl:variable name="targets">
									<xsl:for-each select="/Scripts/Script">
										<xsl:variable name="ant_script_file" select="string(@filtered)"/>
										<xsl:variable name="ant_project_name" select="@name"/>
										<xsl:for-each select="document($ant_script_file)/project/target">
											<xsl:variable name="target_href">
												<xsl:call-template name="_print_project_target_href">
													<xsl:with-param name="ant_script_file" select="$ant_script_file"/>
													<xsl:with-param name="target_name" select="string(@name)"/>
												</xsl:call-template>
											</xsl:variable>
											<Target name="{@name}" href="{$target_href}"/>
										</xsl:for-each>
									</xsl:for-each>
								</xsl:variable>
								<xsl:for-each select="$targets/Target">
									<xsl:sort order="ascending" select="@name"/>
									<font class="FrameItemFont">
										<a href="{@href}" target="{$content_target}">
											<xsl:value-of select="@name"/>
										</a>
									</font>
									<br/>
								</xsl:for-each>
							</td>
						</tr>
					</table>
				</body>
			</html>
		</xsl:result-document>
	</xsl:template>
	<!--
	Prints a targets overview HTML file, which is displayed in the bottom left pane for a given selected project.
	The 'title' parameter should be provided and equal to the HTML title value.
	The 'header' parameter should be provided and equal to the top header of the top header.
	The 'targets' parameter should be provided and equal to a list of the Target elements that should be output;
	a Target element has two attributes: name (the name of the target) and href (the URL of its detail).
	-->
	<xsl:template name="print_targets_overview">
		<xsl:param name="title"/>
		<xsl:param name="header"/>
		<xsl:param name="targets"/>
		<html>
			<head>
				<xsl:call-template name="_print_link_and_meta_head">
					<xsl:with-param name="relative_path" select="'..'"/>
				</xsl:call-template>
				<title>
					<xsl:value-of select="$title"/>
					<xsl:text> Ant targets overview</xsl:text>
				</title>
			</head>
			<body bgcolor="white">
				<font size="+1" class="FrameHeadingFont">
					<b>
						<xsl:value-of select="$header"/>
					</b>
				</font>
				<br/>
				<table border="0" width="100%">
					<tr>
						<td>
							<xsl:for-each select="$targets/Target">
								<xsl:sort order="ascending" select="@name"/>
								<font class="FrameItemFont">
									<a href="{@href}" target="{$content_target}">
										<xsl:value-of select="@name"/>
									</a>
								</font>
								<br/>
							</xsl:for-each>
						</td>
					</tr>
				</table>
			</body>
		</html>
	</xsl:template>
	<!--
	
	The part that concerns XML pretty printing.
	
	-->
	<!-- variable for defining the indentation -->
	<xsl:variable name="xmlout-indendation" select="'&#09;'"/>
	<xsl:variable name="xmlout-carriage" select="'&#13;'"/>
	<!-- templates for outputting prettified XML -->
	<xsl:template match="*" mode="htmlout">
		<xsl:param name="indenter"/>
		<xsl:value-of select="$indenter"/>
		<xsl:text><![CDATA[<]]></xsl:text>
		<xsl:call-template name="_print_ant_task">
			<xsl:with-param name="name" select="name()"/>
		</xsl:call-template>
		<xsl:variable name="multiple_attribute" select="count(@*) > 1"/>
		<xsl:variable name="zero_attribute" select="count(@*) = 0"/>
		<xsl:variable name="attribute_indenter">
			<xsl:choose>
				<xsl:when test="$multiple_attribute">
					<xsl:value-of select="concat($indenter, $xmlout-indendation)"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text disable-output-escaping="yes"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="element_closing_indenter">
			<xsl:choose>
				<xsl:when test="$multiple_attribute">
					<xsl:value-of select="$indenter"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="''"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="$multiple_attribute">
				<xsl:value-of select="$xmlout-carriage"/>
			</xsl:when>
			<xsl:when test="$zero_attribute"/>
			<xsl:otherwise>
				<xsl:text>&#160;</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:for-each select="@*">
			<xsl:apply-templates select="." mode="htmlout">
				<xsl:with-param name="indenter" select="$attribute_indenter"/>
			</xsl:apply-templates>
			<xsl:if test="$multiple_attribute">
				<xsl:value-of select="$xmlout-carriage"/>
			</xsl:if>
		</xsl:for-each>
		<xsl:choose>
			<xsl:when test="* | text()">
				<xsl:value-of select="$element_closing_indenter"/>
				<xsl:text><![CDATA[>]]></xsl:text>
				<xsl:variable name="first" select="count((child::node()[1]|child::comment()[1]|child::processing-instruction()[1])|child::text()[1]) != 1"/>
				<xsl:if test="$first">
					<xsl:value-of select="$xmlout-carriage"/>
				</xsl:if>
				<xsl:apply-templates select="* | text() | comment() | processing-instruction()" mode="htmlout">
					<xsl:with-param name="indenter" select="concat($indenter, $xmlout-indendation)"/>
				</xsl:apply-templates>
				<xsl:variable name="last" select="count(child::node()[last()]|child::text()[last()]) != 1"/>
				<xsl:if test="count(*) > 0">
					<xsl:value-of select="$indenter"/>
				</xsl:if>
				<xsl:text><![CDATA[</]]></xsl:text>
				<xsl:value-of select="name()"/>
				<xsl:text><![CDATA[>]]></xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$element_closing_indenter"/>
				<xsl:text><![CDATA[/>]]></xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="$xmlout-carriage"/>
	</xsl:template>
	<!--Prints an attribute-->
	<xsl:template match="@*" mode="htmlout">
		<xsl:param name="indenter"/>
		<xsl:value-of select="$indenter"/>
		<xsl:value-of select="name()"/>
		<xsl:text>="</xsl:text>
		<xsl:choose>
			<xsl:when test="name() = 'depends' and name(..) = 'target'">
				<xsl:variable name="tokens">
					<xsl:call-template name="_print_tokens">
						<xsl:with-param name="full_string" select="string(.)"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="project" select="/project"/>
				<xsl:for-each select="$tokens/Token">
					<xsl:call-template name="_print_project_target_a">
						<xsl:with-param name="project" select="$project"/>
						<xsl:with-param name="project_name" select="$project/@name"/>
						<xsl:with-param name="target_name" select="string(.)"/>
					</xsl:call-template>
					<xsl:if test="position() != last()">
						<xsl:text>, </xsl:text>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:when test="name() = 'target' and name(..) = 'antcall'">
				<xsl:call-template name="_print_project_target_a">
					<xsl:with-param name="project" select="/project"/>
					<xsl:with-param name="project_name" select="/project/@name"/>
					<xsl:with-param name="target_name" select="string(.)"/>
				</xsl:call-template>
			</xsl:when>
			<!--xsl:when test="name(..) = 'ant'">
				<xsl:choose>
					<xsl:when test="name() = 'antfile'">
						<xsl:call-template name="_print_project_target_a">
							<xsl:with-param name="script_file" select="string(.)"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="name() = 'target'">
						<xsl:call-template name="_print_project_target_a">
							<xsl:with-param name="script_file" select="string(../@antfile)"/>
							<xsl:with-param name="target_name" select="string(.)"/>
						</xsl:call-template>
					</xsl:when>
				</xsl:choose>
			</xsl:when-->
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:text>"</xsl:text>
	</xsl:template>
	<xsl:template match="text()" mode="htmlout">
		<xsl:value-of select="normalize-space(.)"/>
		<xsl:if test="count(following-sibling::node()[1]|following-sibling::*[1]) = 1">
			<xsl:value-of select="$xmlout-carriage"/>
		</xsl:if>
	</xsl:template>
	<xsl:template match="comment()" mode="htmlout">
		<xsl:param name="indenter"/>
		<xsl:value-of select="$indenter"/>
		<xsl:text><![CDATA[<--]]></xsl:text>
		<xsl:value-of select="."/>
		<xsl:text><![CDATA[-->]]></xsl:text>
		<xsl:value-of select="$xmlout-carriage"/>
	</xsl:template>
	<xsl:template match="processing-instruction()" mode="htmlout">
		<xsl:param name="indenter"/>
		<xsl:value-of select="$indenter"/>
		<xsl:text><![CDATA[<?]]></xsl:text>
		<xsl:value-of select="name()"/>
		<xsl:text/>
		<xsl:value-of select="."/>
		<xsl:text><![CDATA[?>]]></xsl:text>
		<xsl:value-of select="$xmlout-carriage"/>
	</xsl:template>
</xsl:stylesheet>
